@@ -1,3 +1,5 @@
+use std::future::Future;
+
 use crate::{Endpoint, Request, Response, Result};
 
 /// Endpoint for the [`and_then`](super::EndpointExt::and_then) method.
@@ -14,12 +16,16 @@ impl<E, F> AndThen<E, F> {
 }
 
 #[async_trait::async_trait]
-impl<E, F> Endpoint for AndThen<E, F>
+impl<E, F, Fut> Endpoint for AndThen<E, F>
 where
     E: Endpoint,
-    F: Fn(Response) -> Result<Response> + Send + Sync + 'static,
+    F: Fn(Response) -> Fut + Send + Sync + 'static,
+    Fut: Future<Output = Result<Response>> + Send + 'static,
 {
     async fn call(&self, req: Request) -> Result<Response> {
-        self.inner.call(req).await.and_then(|resp| (self.f)(resp))
+        match self.inner.call(req).await {
+            Ok(resp) => (self.f)(resp).await,
+            Err(err) => Err(err),
+        }
     }
 }
