@@ -1,3 +1,5 @@
+use std::future::Future;
+
 use crate::{Endpoint, Error, Request, Response, Result};
 
 /// Endpoint for the [`map_err`](super::EndpointExt::map_err) method.
@@ -14,12 +16,16 @@ impl<E, F> MapErr<E, F> {
 }
 
 #[async_trait::async_trait]
-impl<E, F> Endpoint for MapErr<E, F>
+impl<E, F, Fut> Endpoint for MapErr<E, F>
 where
     E: Endpoint,
-    F: Fn(Error) -> Error + Send + Sync + 'static,
+    F: Fn(Error) -> Fut + Send + Sync + 'static,
+    Fut: Future<Output = Error> + Send + 'static,
 {
     async fn call(&self, req: Request) -> Result<Response> {
-        self.inner.call(req).await.map_err(&self.f)
+        match self.inner.call(req).await {
+            Ok(resp) => Ok(resp),
+            Err(err) => Err((self.f)(err).await),
+        }
     }
 }
