@@ -1,16 +1,18 @@
 //! Endpoint related types.
 
 mod and_then;
-mod before;
 #[allow(clippy::module_inception)]
 mod endpoint;
 mod map;
 mod map_err;
 mod map_ok;
+mod map_request;
+mod map_to_response;
 
 pub use and_then::AndThen;
-pub use before::Before;
 pub use endpoint::{Endpoint, EndpointExt};
 pub use map::Map;
 pub use map_err::MapErr;
 pub use map_ok::MapOk;
+pub use map_request::MapRequest;
+pub use map_to_response::MapToResponse;
