@@ -3,9 +3,7 @@ use std::io::ErrorKind;
 use futures_util::TryStreamExt;
 use tokio::io::{AsyncRead, AsyncReadExt};
 
-use crate::{
-    error::ErrorBodyHasBeenTaken, http::header, Body, Error, FromRequest, Request, Result,
-};
+use crate::{http::header, Error, FromRequest, Request, RequestBody, Result};
 
 /// A single field in a multipart stream.
 #[cfg_attr(docsrs, doc(cfg(feature = "multipart")))]
@@ -88,7 +86,7 @@ pub struct Multipart {
 
 #[async_trait::async_trait]
 impl<'a> FromRequest<'a> for Multipart {
-    async fn from_request(req: &'a Request, body: &mut Option<Body>) -> Result<Self> {
+    async fn from_request(req: &'a Request, body: &mut RequestBody) -> Result<Self> {
         let boundary = multer::parse_boundary(
             req.headers()
                 .get(header::CONTENT_TYPE)
@@ -99,9 +97,7 @@ impl<'a> FromRequest<'a> for Multipart {
         .map_err(Error::bad_request)?;
         Ok(Self {
             inner: multer::Multipart::new(
-                tokio_util::io::ReaderStream::new(
-                    body.take().ok_or(ErrorBodyHasBeenTaken)?.into_async_read(),
-                ),
+                tokio_util::io::ReaderStream::new(body.take()?.into_async_read()),
                 boundary,
             ),
         })
@@ -117,3 +113,38 @@ impl Multipart {
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use crate::{handler, Endpoint};
+
+    #[tokio::test]
+    async fn test_multipart_extractor() {
+        #[handler(internal)]
+        async fn index(mut multipart: Multipart) {
+            let field = multipart.next_field().await.unwrap().unwrap();
+            assert_eq!(field.name(), Some("my_text_field"));
+            assert_eq!(field.text().await.unwrap(), "abcd");
+
+            let field = multipart.next_field().await.unwrap().unwrap();
+            assert_eq!(field.name(), Some("my_file_field"));
+            assert_eq!(field.file_name(), Some("a-text-file.txt"));
+            assert_eq!(field.content_type(), Some("text/plain"));
+            assert_eq!(
+                field.text().await.unwrap(),
+                "Hello world\nHello\r\nWorld\rAgain"
+            );
+        }
+
+        let data = "--X-BOUNDARY\r\nContent-Disposition: form-data; name=\"my_text_field\"\r\n\r\nabcd\r\n--X-BOUNDARY\r\nContent-Disposition: form-data; name=\"my_file_field\"; filename=\"a-text-file.txt\"\r\nContent-Type: text/plain\r\n\r\nHello world\nHello\r\nWorld\rAgain\r\n--X-BOUNDARY--\r\n";
+        index
+            .call(
+                Request::builder()
+                    .header("content-type", "multipart/form-data; boundary=X-BOUNDARY")
+                    .body(data),
+            )
+            .await
+            .unwrap();
+    }
+}
